/** Table Header Sorting Background Styles **/
.sorting-none.dark {
    background-image:url("../images/sort_both.png");
    background-repeat: no-repeat;
    background-position: top 6px right;
}

.sorting-asc.dark {
    background-image:url("../images/sort_asc.png");
    background-repeat: no-repeat;
    background-position: top -2px right;
}

.sorting-desc.dark {
    background-image:url("../images/sort_desc.png");
    background-repeat: no-repeat;
    background-position: bottom 1px right;
}

.sorting-none.light {
    background-image:url("../images/sort_both_light.png");
    background-repeat: no-repeat;
    background-position: top 6px right;
}

.sorting-asc.light {
    background-image:url("../images/sort_asc_light.png");
    background-repeat: no-repeat;
    background-position: top -2px right;
}

.sorting-desc.light {
    background-image:url("../images/sort_desc_light.png");
    background-repeat: no-repeat;
    background-position: bottom 1px right;
}

.sortable-th {
    border: 1px solid #ddd;
    cursor: default;
    font-size: 0.9em;
}

.sortable-th .th-container {
    display: block;
    width: 100%;
    vertical-align: middle;
}

.sortable-th .th-container .column-title {
    display: table-cell;
    width: 100%;
    vertical-align: middle;
}

.sortable-th .th-container .sort-container {
    display: table-cell;
    max-width: 19px;
    vertical-align: middle;
    padding-left: 6px;
    height: 32px;
    text-align: center;
}

.sortable-th .th-container .sort-container .sort-controls {
    display: inline-table;
    height: 32px;
    width: 19px;
}

.sortable-th .th-container .sort-container .sort-controls span {
    display:table-cell;
    vertical-align: middle;
    font-family: "Lucida Console", Monaco, monospace;
    font-size: 0.8em;
}

.table thead th.sortable-th,
.table thead th.sfs-th,
.table tbody td.sfs-td {
    vertical-align: middle;
}

/** Machine Event Reason Manager pagination styles **/
.page-item.page-of .page-link {
    color: #000000;
    background-color: #ffffff;
    border-color: #dddddd;
    padding-top: 0;
    padding-bottom: 0;
}

input#page_number_text {
    height: 36px;
    width: 40px;
    display: inline-block;
}

/** Specific Styles **/
#clear_sorts_button {
    padding-bottom: 0.1rem;
}

#select_all_pages_checkbox {
    margin: -2px 0 0 6px;
}

label[for="select_all_pages_checkbox"] {
    margin-bottom: 0;
}

#paginate_by_select {
    width: 150px;
    flex: none;
}

#search_group {
    max-width: 400px;
}

#hidden_filters_message_div {
    display: none;
    color: red;
}

#select_all_objects_checkbox {
    margin: auto;
}

.row#select_per_page_and_search_row, .row#filters_row {
    margin-bottom: 10px;
}

.object-list-checkbox {
    margin: 0 auto;
}

.sort-controls span strong {
    display: none;
}

#apply_and_clear_filters_buttons {
    width: 207px;
}

div.filter-header {
    white-space: nowrap;
    overflow: hidden;
    text-overflow: ellipsis;
    max-width: 100%;
    font-size: 1em;
    font-weight: bolder;
}

/** Select Filters Accordion Styles **/
a.select-filters {
    color: #000000
}

a.select-filters:hover {
    text-decoration: none;
    color: #000000;
}

a.select-filters.fa-caret-down {
    transition: .3s transform ease-in-out;
}

a.select-filters.collapsed .fa-caret-down {
    transform: rotate(-90deg);
}

/** Fontawesome styles **/
.svg-inline--fa {
    margin-right: 0.3em;
}

.svg-inline--fa.fa-fw {
    margin-right: 0.2em;
}

.svg-inline--fa.no-right-margin {
    margin-right: 0;
}

.button-default-bg-color {
    background-color: #DDDDDD;
}
